@@ -283,7 +283,7 @@ html_engine_load_done_cb (HTMLEngine *engine, gpointer data)
 static void
 html_engine_url_requested_cb (HTMLEngine *engine,
 			      const gchar *url,
-			      GtkHTMLStreamHandle handle,
+			      GtkHTMLStream *handle,
 			      gpointer data)
 {
 	GtkHTML *gtk_html;
@@ -329,16 +329,19 @@ html_engine_submit_cb (HTMLEngine *engine,
 	gtk_signal_emit (GTK_OBJECT (gtk_html), signals[SUBMIT], method, url, encoding);
 }
 
-static void
+static gboolean
 html_engine_object_requested_cb (HTMLEngine *engine,
 		       GtkHTMLEmbedded *eb,
 		       gpointer data)
 {
 	GtkHTML *gtk_html;
+	gboolean ret_val = FALSE;
 
 	gtk_html = GTK_HTML (data);
 
-	gtk_signal_emit (GTK_OBJECT (gtk_html), signals[OBJECT_REQUESTED], eb);
+	ret_val = FALSE;
+	gtk_signal_emit (GTK_OBJECT (gtk_html), signals[OBJECT_REQUESTED], eb, &ret_val);
+	return ret_val;
 }
 
 
@@ -1123,11 +1126,11 @@ class_init (GtkHTMLClass *klass)
 
 	signals [OBJECT_REQUESTED] =
 		gtk_signal_new ("object_requested",
-				GTK_RUN_FIRST,
+				GTK_RUN_LAST,
 				object_class->type,
 				GTK_SIGNAL_OFFSET (GtkHTMLClass, object_requested),
-				gtk_marshal_NONE__POINTER,
-				GTK_TYPE_NONE, 1,
+				gtk_marshal_BOOL__POINTER,
+				GTK_TYPE_BOOL, 1,
 				GTK_TYPE_OBJECT);
 	
 	signals [CURRENT_PARAGRAPH_STYLE_CHANGED] =
@@ -1323,12 +1326,12 @@ gtk_html_allow_selection (GtkHTML *html,
 }
 
 
-GtkHTMLStreamHandle
-gtk_html_begin (GtkHTML *html, const gchar *url)
+GtkHTMLStream *
+gtk_html_begin (GtkHTML *html)
 {
-	GtkHTMLStreamHandle *handle;
+	GtkHTMLStream *handle;
 
-	handle = html_engine_begin (html->engine, url);
+	handle = html_engine_begin (html->engine);
 	if (handle == NULL)
 		return NULL;
 
@@ -1341,7 +1344,7 @@ gtk_html_begin (GtkHTML *html, const gchar *url)
 
 void
 gtk_html_write (GtkHTML *html,
-		GtkHTMLStreamHandle handle,
+		GtkHTMLStream *handle,
 		const gchar *buffer,
 		size_t size)
 {
@@ -1350,7 +1353,7 @@ gtk_html_write (GtkHTML *html,
 
 void
 gtk_html_end (GtkHTML *html,
-	      GtkHTMLStreamHandle handle,
+	      GtkHTMLStream *handle,
 	      GtkHTMLStreamStatus status)
 {
 	gtk_html_stream_close (handle, status);
@@ -1378,7 +1381,7 @@ gtk_html_jump_to_anchor (GtkHTML *html,
 {
 	g_return_val_if_fail (html != NULL, FALSE);
 	g_return_val_if_fail (GTK_IS_HTML (html), FALSE);
-
+	
 	return html_engine_goto_anchor (html->engine, anchor);
 }
 
@@ -1391,10 +1394,30 @@ gtk_html_save (GtkHTML *html,
 	g_return_val_if_fail (html != NULL, FALSE);
 	g_return_val_if_fail (GTK_IS_HTML (html), FALSE);
 	g_return_val_if_fail (receiver != NULL, FALSE);
-
+	
 	return html_engine_save (html->engine, receiver, data);
 }
 
+gboolean
+gtk_html_export (GtkHTML *html,
+		 const char *type,
+		 GtkHTMLSaveReceiverFn receiver,
+		 gpointer data)
+{
+	g_return_val_if_fail (html != NULL, FALSE);
+	g_return_val_if_fail (GTK_IS_HTML (html), FALSE);
+	g_return_val_if_fail (receiver != NULL, FALSE);
+	
+	if (strcmp (type, "text/html") == 0) {
+		return html_engine_save (html->engine, receiver, data);
+	} else if (strcmp (type, "text/plain") == 0) {
+		return html_engine_save_plain (html->engine, receiver,
+					       data);  
+	} else {
+		return FALSE;
+	}
+}
+
 
 void
 gtk_html_private_calc_scrollbars (GtkHTML *html)
@@ -1490,7 +1513,7 @@ gtk_html_set_paragraph_style (GtkHTML *html,
 		return;
 
 	if (! html_engine_set_clueflow_style (html->engine, clueflow_style, 0, 0,
-					      HTML_ENGINE_SET_CLUEFLOW_STYLE))
+					      HTML_ENGINE_SET_CLUEFLOW_STYLE, TRUE))
 		return;
 
 	html->paragraph_style = style;
@@ -1507,7 +1530,7 @@ gtk_html_indent (GtkHTML *html,
 	g_return_if_fail (GTK_IS_HTML (html));
 
 	html_engine_set_clueflow_style (html->engine, 0, 0, delta,
-					HTML_ENGINE_SET_CLUEFLOW_INDENTATION);
+					HTML_ENGINE_SET_CLUEFLOW_INDENTATION, TRUE);
 
 	update_styles (html);
 }
@@ -1535,7 +1558,7 @@ gtk_html_align_paragraph (GtkHTML *html,
 	align = paragraph_alignment_to_html (alignment);
 
 	html_engine_set_clueflow_style (html->engine, 0, align, 0,
-					HTML_ENGINE_SET_CLUEFLOW_ALIGNMENT);
+					HTML_ENGINE_SET_CLUEFLOW_ALIGNMENT, TRUE);
 }
 
 
