@@ -466,10 +466,9 @@ calc_size (HTMLObject *o,
 			/* By setting "newLine = true" we move the complete run
 			   to a new line.  We shouldn't set newLine if we are
 			   at the start of a line.  */
-
 			runWidth = 0;
 			run = obj;
-
+			
 			while ( run
 				&& ! (run->flags & HTML_OBJECT_FLAG_SEPARATOR)
 				&& ! (run->flags & HTML_OBJECT_FLAG_NEWLINE)
@@ -478,14 +477,12 @@ calc_size (HTMLObject *o,
 				HTMLVAlignType valign;
 				gint width_left;
 
-				run->max_width = rmargin - lmargin;
-
 				if (is_pre)
 					width_left = -1;
 				else
 					width_left = rmargin - runWidth - w;
 
-				if (! is_pre && run != obj && runWidth+run->nb_width > rmargin - lmargin)
+				if (! is_pre && run != obj && runWidth + run->nb_width > rmargin - lmargin)
 					break;
 
 				fit = html_object_fit_line (run,
@@ -546,15 +543,10 @@ calc_size (HTMLObject *o,
 					break;
 				}
 
-				if (! is_pre && runWidth > rmargin - lmargin)
+				if (! is_pre && (runWidth > rmargin - lmargin))
 					break;
 			}
 
-			/* If these objects do not fit in the current line and we are
-			   not at the start of a line then end the current line in
-			   preparation to add this run in the next pass. */
-			if (! is_pre && w > lmargin && w + runWidth > rmargin)
-				newLine = TRUE;
 
 			if (! newLine) {
 				gint new_y, new_lmargin, new_rmargin;
@@ -602,12 +594,14 @@ calc_size (HTMLObject *o,
 				} else {
 					while (obj != run) {
 						if (obj->x != w) {
-							obj->x = w;
+							obj->x = w;   
 							changed = TRUE;
 						}
 						w += obj->width;
 						obj = obj->next;
 					}
+					/* we've used up this line so insert a newline */
+					newLine = TRUE;
 				}
 				lmargin = html_object_get_left_margin (o->parent, o->y);
 				
@@ -730,12 +724,18 @@ calc_size (HTMLObject *o,
 			
 			newLine = FALSE;
 			clear = HTML_CLEAR_NONE;
+		
 		}
 	}
 	
+       
 	if (o->width < o->max_width)
 		o->width = o->max_width;
 
+#if 1
+	if (o->width > rmargin - o->x)
+		o->width = rmargin - o->x;
+#endif
 	add_post_padding (HTML_CLUEFLOW (o), padding);
 
 	if (o->ascent != old_ascent || o->descent != old_descent || o->width != old_width)
@@ -911,9 +911,7 @@ write_indentation_tags_helper (HTMLEngineSaveState *state,
 					return FALSE;
 			}
 
-			if (! html_engine_save_output_string (state, "<")
-			    || ! html_engine_save_output_string (state, tag)
-			    || ! html_engine_save_output_string (state, ">\n"))
+			if (! html_engine_save_output_string (state, "<%s>\n", tag))
 				return FALSE;
 		}
 	} else {
@@ -925,9 +923,7 @@ write_indentation_tags_helper (HTMLEngineSaveState *state,
 				}
 			}
 
-			if (! html_engine_save_output_string (state, "</")
-			    || ! html_engine_save_output_string (state, tag)
-			    || ! html_engine_save_output_string (state, ">\n"))
+			if (! html_engine_save_output_string (state, "</%s>\n", tag))
 				return FALSE;
 		}
 	}
@@ -987,6 +983,21 @@ get_tag_for_style (const HTMLClueFlow *flow)
 	}
 }
 
+static const gchar *
+get_tag_for_item_group (const HTMLClueFlow *flow)
+{
+	switch (flow->style) {
+	case HTML_CLUEFLOW_STYLE_ITEMDOTTED:
+		return "UL";
+	case HTML_CLUEFLOW_STYLE_ITEMROMAN:
+	case HTML_CLUEFLOW_STYLE_ITEMDIGIT:
+		return "OL";
+	default:
+		g_warning ("Unknown HTMLClueFlowStyle %d", flow->style);
+		return NULL;
+	}
+}
+
 static const gchar *
 halign_to_string (HTMLHAlignType halign)
 {
@@ -1005,11 +1016,11 @@ halign_to_string (HTMLHAlignType halign)
 static gboolean 
 is_similar (HTMLObject *self, HTMLObject *friend)
 {
-	
 	if (friend &&  HTML_OBJECT_TYPE (friend) == HTML_TYPE_CLUEFLOW) {
 		if ((HTML_CLUEFLOW (friend)->style == HTML_CLUEFLOW (self)->style)
-		    && (HTML_CLUEFLOW (friend)->level == HTML_CLUEFLOW (self)->level))
-			return TRUE;
+		    && (HTML_CLUEFLOW (friend)->level == HTML_CLUEFLOW (self)->level)) {
+				return TRUE;
+		}
 	}
 	return FALSE;
 }
@@ -1026,17 +1037,6 @@ save (HTMLObject *self,
 	clueflow = HTML_CLUEFLOW (self);
 	halign = HTML_CLUE (self)->halign;
 
-	if (self->prev != NULL
-	    && HTML_OBJECT_TYPE (HTML_CLUE (self)->tail) != HTML_TYPE_RULE
-	    && HTML_CLUEFLOW (self)->style != HTML_CLUEFLOW_STYLE_PRE) {
-		/* This is a nasty hack: the rule takes all of the space, so we
-                   don't want it to create a new newline if it's the last
-                   element on the paragraph.  Also, everything but the "normal"
-                   paragraph style would get an extra newline if we add a
-                   `<BR>'.  */
-		html_engine_save_output_string (state, "<BR>\n");
-	}
-
 	if (! write_indentation_tags (clueflow, state))
 		return FALSE;
 
@@ -1048,19 +1048,35 @@ save (HTMLObject *self,
 	if (is_similar (self, self->next))
 		end = FALSE;
 	
+	if (self->prev != NULL && ! is_item (clueflow)
+	    && HTML_OBJECT_TYPE (HTML_CLUE (self)->tail) != HTML_TYPE_RULE) {
+		if (HTML_CLUEFLOW (self)->style != HTML_CLUEFLOW_STYLE_PRE) {
+			/* This is a nasty hack: the rule takes all of the space, so we
+			   don't want it to create a new newline if it's the last
+			   element on the paragraph.  Also, everything but the "normal"
+			   paragraph style would get an extra newline if we add a
+			   `<BR>'.  */
+			html_engine_save_output_string (state, "<BR>\n");
+		} else {
+			html_engine_save_output_string (state, "\n");
+		}
+	}
+
 	/* Alignment tag.  */
 	if (halign != HTML_HALIGN_NONE && halign != HTML_HALIGN_LEFT) {
-		if (! html_engine_save_output_string (state, "<DIV ALIGN=")
-		    || ! html_engine_save_output_string (state, halign_to_string (halign))
-		    || ! html_engine_save_output_string (state, ">"))
+		if (! html_engine_save_output_string (state, "<DIV ALIGN=%s>\n", halign_to_string (halign)))
 			return FALSE;
 	}
 
+	if (start && is_item (clueflow)) {
+		if (! html_engine_save_output_string (state, "<%s>\n",
+						      get_tag_for_item_group (HTML_CLUEFLOW (self))))
+			return FALSE;
+	}		
+
 	/* Start tag.  */
-	if (tag != NULL && start
-	    && (! html_engine_save_output_string (state, "<")
-		|| ! html_engine_save_output_string (state, tag)
-		|| ! html_engine_save_output_string (state, ">")))
+	if (tag != NULL && (start || is_item (clueflow))
+	    && (! html_engine_save_output_string (state, "<%s>", tag)))
 		return FALSE;
 
 	/* Paragraph's content.  */
@@ -1068,13 +1084,17 @@ save (HTMLObject *self,
 		return FALSE;
 
 	/* End tag.  */
-	if (tag && end) {
-		if (! html_engine_save_output_string (state, "</")
-		    || ! html_engine_save_output_string (state, tag)
-		    || ! html_engine_save_output_string (state, ">"))
+	if (tag && (end || is_item (clueflow))) {
+		if (! html_engine_save_output_string (state, "</%s>", tag))
 			return FALSE;
 	}
 
+	if (end && is_item (clueflow)) {
+		if (! html_engine_save_output_string (state, "\n</%s>",
+						      get_tag_for_item_group (HTML_CLUEFLOW (self))))
+			return FALSE;
+	}		
+
 	/* Close alignment tag.  */
 	if (halign != HTML_HALIGN_NONE && halign != HTML_HALIGN_LEFT) {
 		if (! html_engine_save_output_string (state, "</DIV>\n"))
@@ -1087,6 +1107,21 @@ save (HTMLObject *self,
 	return TRUE;
 }
 
+static gboolean
+save_plain (HTMLObject *self,
+	    HTMLEngineSaveState *state)
+{
+
+	/* Paragraph's content.  */
+	if (! HTML_OBJECT_CLASS (&html_clue_class)->save_plain (self, state))
+		return FALSE;
+
+	if (!html_engine_save_output_string (state, "\n"))
+		return FALSE;
+	
+	return TRUE;
+}
+
 
 static gint
 check_page_split (HTMLObject *self,
@@ -1427,6 +1462,7 @@ html_clueflow_class_init (HTMLClueFlowClass *klass,
 	object_class->calc_preferred_width = calc_preferred_width;
 	object_class->draw = draw;
 	object_class->save = save;
+	object_class->save_plain = save_plain;
 	object_class->check_page_split = check_page_split;
 	object_class->check_point = check_point;
 	object_class->append_selection_string = append_selection_string;
