@@ -55,9 +55,13 @@ draw (HTMLObject *o,
       gint width, gint height,
       gint tx, gint ty)
 {
-
 	HTMLEmbedded *element = HTML_EMBEDDED(o);
 	gint new_x, new_y;
+	ArtIRect paint;
+
+	html_object_calc_intersection (o, &paint, x, y, width, height);
+	if (art_irect_empty (&paint))
+		return;
 
 	if (element->widget) {
 
@@ -74,10 +78,12 @@ draw (HTMLObject *o,
 			
 			gtk_layout_move(GTK_LAYOUT(element->parent), element->widget,
 					new_x, new_y);
+		} else {
+			gtk_widget_queue_draw (element->widget);
 		}
 		element->abs_x = new_x;
 		element->abs_y = new_y;
-		gtk_widget_queue_draw (element->widget);
+
 	}
 }
 
@@ -327,9 +333,11 @@ html_embedded_size_recalc(HTMLEmbedded *em)
 
 	o = HTML_OBJECT (em);
 
-	eb = (GtkHTMLEmbedded *)em->widget;
+	if (em->widget == NULL)
+		return;
+
+	gtk_widget_size_request(em->widget, &req);
 
-	gtk_widget_size_request((GtkWidget *)eb, &req);
 	o->width = req.width;
 	if (GTK_IS_HTML_EMBEDDED(eb)) {
 		o->descent = eb->descent;
