@@ -26,7 +26,7 @@
 
 #include "htmlengine-edit-fontstyle.h"
 
-#define PARANOID_DEBUG
+/* #define PARANOID_DEBUG */
 
 
 struct _FontStyleSegment {
@@ -105,6 +105,9 @@ merge_safely (HTMLText *text,
 	      HTMLCursor *cursor,
 	      HTMLCursor *mark)
 {
+	html_text_master_destroy_slaves (HTML_TEXT_MASTER (text));
+	html_text_master_destroy_slaves (HTML_TEXT_MASTER (other));
+
 	update_cursor_for_merge (text, other, prepend, cursor);
 	if (mark != NULL)
 		update_cursor_for_merge (text, other, prepend, mark);
@@ -164,7 +167,8 @@ merge_backward (HTMLObject *object,
 		if (HTML_OBJECT_TYPE (p) == HTML_TYPE_TEXTSLAVE) {
 			html_clue_remove (HTML_CLUE (p->parent), p);
 			html_object_destroy (p);
-		} else if (HTML_OBJECT_TYPE (object) == HTML_OBJECT_TYPE (p)
+		} else if (html_object_is_text (p)
+			   && HTML_OBJECT_TYPE (object) == HTML_OBJECT_TYPE (p)
 			   && html_text_check_merge (HTML_TEXT (object), HTML_TEXT (p))) {
 			total_merge += HTML_TEXT (object)->text_len;
 			merge_safely (HTML_TEXT (object), HTML_TEXT (p), TRUE, cursor, mark);
@@ -193,7 +197,8 @@ merge_forward (HTMLObject *object,
 		if (HTML_OBJECT_TYPE (p) == HTML_TYPE_TEXTSLAVE) {
 			html_clue_remove (HTML_CLUE (p->parent), p);
 			html_object_destroy (p);
-		} else if (HTML_OBJECT_TYPE (object) == HTML_OBJECT_TYPE (p)
+		} else if (html_object_is_text (p)
+			   && HTML_OBJECT_TYPE (object) == HTML_OBJECT_TYPE (p)
 			   && html_text_check_merge (HTML_TEXT (object), HTML_TEXT (p))) {
 			total_merge += HTML_TEXT (object)->text_len;
 			merge_safely (HTML_TEXT (object), HTML_TEXT (p), FALSE, cursor, mark);
@@ -249,18 +254,22 @@ set_font_style (HTMLEngine *engine,
 		GtkHTMLFontStyle or_mask,
 		guint count,
 		gboolean backwards,
-		gboolean do_undo)
+		gboolean want_undo)
 {
 	GtkHTMLFontStyle font_style;
 	GtkHTMLFontStyle last_font_style;
 	HTMLObject *p, *pparent;
+	HTMLObject *previous_parent;
 	GList *orig_styles;
 	guint total_merged;
 
+	g_print ("%s -- setting %d elements.\n", __FUNCTION__, count);
+
 	p = engine->cursor->object;
 	last_font_style = GTK_HTML_FONT_STYLE_DEFAULT;
 	orig_styles = NULL;
 
+	previous_parent = NULL;
 	while (count > 0) {
 		HTMLTextMaster *curr;
 		guint start, end;
@@ -295,6 +304,7 @@ set_font_style (HTMLEngine *engine,
 							       engine->cursor, engine->mark));
 			g_print ("Splitting at start %p -> %p\n", p, curr);
 			html_clue_append_after (HTML_CLUE (p->parent), HTML_OBJECT (curr), p);
+			p = HTML_OBJECT (curr);
 			end -= start;
 			start = 0;
 		}
@@ -311,7 +321,7 @@ set_font_style (HTMLEngine *engine,
 		}
 
 		/* Save the original style for undo.  */
-		if (do_undo)
+		if (want_undo)
 			prepend_style_segment (&orig_styles,
 					       HTML_TEXT (curr)->font_style,
 					       end - start);
@@ -340,6 +350,10 @@ set_font_style (HTMLEngine *engine,
 				count -= total_merged;
 		}
 
+		/* FIXME slow slow slow!  */
+		html_object_relayout (p->parent->parent, engine, p->parent);
+		html_engine_queue_draw (engine, p->parent);
+
 		if (count == 0)
 			break;
 
@@ -373,17 +387,24 @@ set_font_style (HTMLEngine *engine,
 			merge_forward (p, engine->cursor, engine->mark);
 	}
 
-	if (do_undo)
-		return g_list_reverse (orig_styles);
-	else
+	html_object_relayout (p->parent->parent, engine, p->parent);
+	html_engine_queue_draw (engine, p->parent);
+
+	html_cursor_normalize (engine->cursor);
+
+	if (! want_undo)
 		return NULL;
+
+	orig_styles = g_list_reverse (orig_styles);
+
+	return orig_styles;
 }
 
 static void
 set_font_style_in_selection (HTMLEngine *engine,
 			     GtkHTMLFontStyle and_mask,
 			     GtkHTMLFontStyle or_mask,
-			     gboolean do_undo)
+			     gboolean want_undo)
 {
 	gboolean backwards;
 	GList *orig_styles;
@@ -411,14 +432,14 @@ set_font_style_in_selection (HTMLEngine *engine,
 		backwards = FALSE;
 	}
 
-	orig_styles = set_font_style (engine, and_mask, or_mask, count, backwards, do_undo);
+	orig_styles = set_font_style (engine, and_mask, or_mask, count, backwards, want_undo);
 
 #ifdef PARANOID_DEBUG
 	g_print ("Tree after changing font style:\n");
 	gtk_html_debug_dump_tree_simple (engine->clue, 2);
 #endif
 
-	if (! do_undo)
+	if (! want_undo)
 		return;
 
 	setup_undo (engine,
@@ -490,6 +511,79 @@ setup_redo (HTMLEngine *engine,
 	html_undo_add_redo_action (engine->undo, action);
 }
 
+static gboolean
+move_to_next_text_segment_forwards (HTMLEngine *engine)
+{
+	HTMLCursor *cursor;
+	gboolean retval;
+
+	cursor = engine->cursor;
+
+	while (1) {
+		guint object_length;
+
+		/* Check if there is a text element on the immediate right of the cursor.  */
+
+		if (html_object_is_text (cursor->object))
+			object_length = HTML_TEXT (cursor->object)->text_len;
+		else
+			object_length = 1;
+
+		if (cursor->offset == object_length) {
+			HTMLObject *next;
+
+			next = html_object_next_not_slave (cursor->object);
+			if (next != NULL && html_object_is_text (next)) {
+				retval = TRUE;
+				break;
+			}
+		} else if (html_object_is_text (cursor->object)) {
+			retval = TRUE;
+			break;
+		}
+
+		if (! html_engine_move_cursor (engine, HTML_ENGINE_CURSOR_RIGHT, 1)) {
+			retval = FALSE;
+			break;
+		}
+	}
+
+	return retval;
+}
+
+static gboolean
+move_to_next_text_segment_backwards (HTMLEngine *engine)
+{
+	HTMLCursor *cursor;
+	gboolean retval;
+
+	cursor = engine->cursor;
+
+	while (1) {
+		/* Check if there is a text element on the immediate left of the cursor.  */
+
+		if (cursor->offset == 0) {
+			HTMLObject *prev;
+
+			prev = html_object_prev_not_slave (cursor->object);
+			if (prev != NULL && html_object_is_text (prev)) {
+				retval = TRUE;
+				break;
+			}
+		} else if (html_object_is_text (cursor->object)) {
+			retval = TRUE;
+			break;
+		}
+
+		if (! html_engine_move_cursor (engine, HTML_ENGINE_CURSOR_LEFT, 1)) {
+			retval = FALSE;
+			break;
+		}
+	}
+
+	return retval;
+}
+
 static void
 do_undo (HTMLEngine *engine,
 	 gpointer closure)
@@ -508,10 +602,17 @@ do_undo (HTMLEngine *engine,
 				segment->style, segment->count,
 				data->backwards, FALSE);
 
-		if (data->backwards)
-			html_engine_move_cursor (engine, HTML_ENGINE_CURSOR_LEFT, segment->count);
-		else
-			html_engine_move_cursor (engine, HTML_ENGINE_CURSOR_RIGHT, segment->count);
+		if (data->backwards) {
+			html_engine_move_cursor (engine, HTML_ENGINE_CURSOR_LEFT,
+						 segment->count);
+			if (! move_to_next_text_segment_backwards (engine))
+				break;
+		} else {
+			html_engine_move_cursor (engine, HTML_ENGINE_CURSOR_RIGHT,
+						 segment->count);
+			if (! move_to_next_text_segment_forwards (engine))
+				break;
+		}
 	}
 
 	setup_redo (engine, data);
