@@ -25,10 +25,14 @@
 #include "htmlengine.h"
 
 /* Insertion operations.  */
-void   html_engine_insert_para  (HTMLEngine  *e,
-				 gboolean     vspace);
 guint  html_engine_insert       (HTMLEngine  *e,
 				 const gchar *text,
 				 guint        len);
 
+/* Links */
+
+void  html_engine_insert_link (HTMLEngine  *e,
+			       const gchar *text,
+			       const gchar *href);
+
 #endif /* _HTMLENGINE_EDIT_INSERT_H */
