@@ -25,6 +25,7 @@
 #include "htmltextmaster.h"
 
 #include "htmlengine-cutbuffer.h"
+#include "htmlengine-edit-clueflowstyle.h"
 #include "htmlengine-edit-cut.h"
 #include "htmlengine-edit-movement.h"
 #include "htmlengine-edit-delete.h"
@@ -33,7 +34,7 @@
 
 #include "htmlengine-edit-paste.h"
 
-#define PARANOID_DEBUG
+/* #define PARANOID_DEBUG */
 
 
 /* This function adds an empty HTMLTextMaster object to @flow; it is
@@ -87,31 +88,30 @@ split_at_cursor (HTMLEngine *engine)
 
 	cursor = engine->cursor;
 
-	/* If we are at the beginning of the element, we can always simply
-           prepend stuff to the element itself.  */
+	/* If we are at the beginning of the element, we can always simply prepend stuff
+           to the element itself.  */
 	if (cursor->offset == 0)
 		return FALSE;
 
 	current = cursor->object;
 
 	if (html_object_is_text (current)) {
-		/* If we are at the end of a text element, we just have to
-                   append elements.  */
+		/* If we are at the end of a text element, we just have to append
+                   elements.  */
 		if (cursor->offset >= HTML_TEXT (current)->text_len)
 			return TRUE;
 
-		/* Bad luck, we have to split at the current position.  After
-                   calling `split_text()', the cursor will be positioned at the
-                   beginning of the second half of the split text, so we just
-                   have to prepend the new elements there.  */
+		/* Bad luck, we have to split at the current position.  After calling
+                   `split_text()', the cursor will be positioned at the beginning of the
+                   second half of the split text, so we just have to prepend the new
+                   elements there.  */
 
 		split_text (engine);
 		return FALSE;
 	}
 
-	/* We are not in a text element, so no splitting is needed.
-           If we are at the end of the non-texte element, we must
-           append; otherwise, we must prepend.  */
+	/* We are not in a text element, so no splitting is needed.  If we are at the end
+           of the non-text element, we must append; otherwise, we must prepend.  */
 	if (cursor->offset == 0)
 		return FALSE;
 	else
@@ -154,7 +154,8 @@ split_first_clueflow_at_cursor (HTMLEngine *engine,
 	if (curr_clue->prev == NULL)
 		html_clue_prepend (HTML_CLUE (curr_clue->parent), new_clueflow);
 	else
-		html_clue_append_after (HTML_CLUE (curr_clue->parent), new_clueflow, curr_clue->prev);
+		html_clue_append_after (HTML_CLUE (curr_clue->parent), new_clueflow,
+					curr_clue->prev);
 
 	/* Move the stuff until the cursor position into the new HTMLClueFlow.  */
 
@@ -202,7 +203,7 @@ prepare_clueflows (HTMLEngine *engine,
 	g_return_val_if_fail (curr->parent != NULL, FALSE);
 	g_return_val_if_fail (HTML_OBJECT_TYPE (curr->parent) == HTML_TYPE_CLUEFLOW, FALSE);
 
-	clue = NULL;		/* Make compiler happy.  */
+	clue = NULL;
 
 	first = TRUE;
 	retval = FALSE;
@@ -221,12 +222,13 @@ prepare_clueflows (HTMLEngine *engine,
 
 			first = FALSE;
 			retval = TRUE;
-		} else {
-			if (first)
+		} else {	/* ! first || append */
+			if (first) { /* first && append */
 				clue = engine->cursor->object->parent;
+				first = FALSE;
+			}
 
 			g_assert (clue != NULL);
-
 			clue = add_new_clueflow (engine, HTML_CLUEFLOW (obj), clue);
 		}
 	}
@@ -324,6 +326,60 @@ remove_slaves_at_cursor (HTMLEngine *engine)
 	html_clueflow_remove_text_slaves (HTML_CLUEFLOW (parent));
 }
 
+static void
+move_cursor_to_safe_object (HTMLEngine *engine,
+			    HTMLObject *object)
+{
+	HTMLObject *prev;
+	HTMLObject *next;
+
+	next = html_object_next_not_slave (object);
+	if (next != NULL) {
+		engine->cursor->object = next;
+		engine->cursor->offset = 0;
+
+		return;
+	}
+
+	prev = html_object_prev_not_slave (object);
+	if (prev != NULL) {
+		engine->cursor->object = prev;
+
+		if (html_object_is_text (prev))
+			engine->cursor->offset = HTML_TEXT (prev)->text_len;
+		else
+			engine->cursor->offset = 1;
+
+		return;
+	}
+
+	g_assert_not_reached ();
+}
+
+static void
+remove_element_if_empty_text (HTMLEngine *engine,
+			      HTMLObject *object)
+{
+	if (object == NULL)
+		return;
+	if (! html_object_is_text (object))
+		return;
+	if (HTML_TEXT (object)->text_len != 0)
+		return;
+
+	if (engine->cursor->object == object) {
+		/* Of course, if we are destroying this object, it must be empty and
+                   consequently, if the cursor is in it, its offset into the object must
+                   be zero.  */
+		g_assert (engine->cursor->offset == 0);
+		move_cursor_to_safe_object (engine, object);
+	}
+
+	html_clue_remove (HTML_CLUE (object->parent), object);
+	html_text_master_destroy_slaves (HTML_TEXT_MASTER (object));
+	html_object_destroy (object);
+}
+
 
 static guint
 do_paste (HTMLEngine *engine,
@@ -340,7 +396,8 @@ do_paste (HTMLEngine *engine,
 	gtk_html_debug_dump_list_simple (buffer, 1);
 #endif
 
-	/* 1. Freeze the engine.  */
+	/* 1. Freeze the engine.  FIXME: It's very inefficient to
+           freeze/thaw at every paste operation.  */
 	html_engine_freeze (engine);
 
 #ifdef PARANOID_DEBUG
@@ -379,7 +436,7 @@ do_paste (HTMLEngine *engine,
 		HTMLObject *obj_copy;
 		HTMLObject *obj;
 
-		obj = (HTMLObject *) p->data;
+		obj = HTML_OBJECT (p->data);
 
 		if (HTML_OBJECT_TYPE (obj) != HTML_TYPE_CLUEFLOW) {
 #ifdef PARANOID_DEBUG
@@ -395,57 +452,41 @@ do_paste (HTMLEngine *engine,
 				prepend_object (engine, obj_copy);
 		} else {
 			HTMLObject *clueflow;
+
 #ifdef PARANOID_DEBUG
 			g_print ("*** Pasting HTMLClueFlow\n");
 #endif
 
-			/* This is an HTMLClueFlow, which we have already added to the
-                           tree: move to the next element in the cut buffer, and insert it
-                           into the next HTMLClueFlow (which is the one we created).  If
-                           we get another ClueFlow after this one, we need to fill the
-                           empty HTMLClueFlow with an empty text element.  */
-
-			if (!p->next) {
-				HTMLClueFlow *next;
-
-				g_assert (HTML_OBJECT_TYPE (engine->cursor->object->parent->next) == HTML_TYPE_CLUEFLOW);
-
-				next = HTML_CLUEFLOW (engine->cursor->object->parent->next);
-				if (!HTML_CLUE (next)->head)
-					add_empty_text_master_to_clueflow (next);
-				engine->cursor->object = HTML_CLUE (next)->head;
-				engine->cursor->offset = 0;
-				engine->cursor->position++;
-#ifdef PARANOID_DEBUG
-				g_print ("    next is NULL: bailing out\n");
-#endif
-				break;
-			}
-			p = p->next;
-
-			obj = HTML_OBJECT (p->data);
-
 			clueflow = engine->cursor->object->parent->next;
 			g_assert (clueflow != NULL);
 			g_assert (HTML_OBJECT_TYPE (clueflow) == HTML_TYPE_CLUEFLOW);
 
-			while (HTML_OBJECT_TYPE (obj) == HTML_TYPE_CLUEFLOW) {
+			while (p->next != NULL
+			       && HTML_OBJECT_TYPE (p->next->data) == HTML_TYPE_CLUEFLOW) {
 				add_empty_text_master_to_clueflow (HTML_CLUEFLOW (clueflow));
-				engine->cursor->position += 2;
-				/* engine->cursor->object    = HTML_CLUE (clueflow)->head;
-				   engine->cursor->offset    = 0; */
-				clueflow = clueflow->next;
+
+				engine->cursor->position ++;
+
+				if (p->next == NULL)
+					break;
 
 				p = p->next;
-				if (p == NULL)
-					break;
-				obj = HTML_OBJECT (p->data);
+				clueflow = clueflow->next;
 			}
 
-			if (p == NULL)
-				break;
+			engine->cursor->position ++;
 
-			obj_copy = html_object_dup (obj);
+			if (p->next == NULL) {
+				if (HTML_CLUE (clueflow)->head == NULL)
+					add_empty_text_master_to_clueflow (HTML_CLUEFLOW (clueflow));
+				engine->cursor->object = HTML_CLUE (clueflow)->head;
+				engine->cursor->offset = 0;
+				break;
+			}
+
+			p = p->next;
+
+			obj_copy = html_object_dup (HTML_OBJECT (p->data));
 			html_object_change_set (obj_copy, HTML_CHANGE_ALL);
 
 			html_clue_prepend (HTML_CLUE (clueflow), obj_copy);
@@ -455,18 +496,28 @@ do_paste (HTMLEngine *engine,
 			engine->cursor->position += obj_len (obj_copy);
 
 			append = TRUE;
+
 #ifdef PARANOID_DEBUG
 			g_print ("*** Appending %s to %p\n",
 				 html_type_name (HTML_OBJECT_TYPE (obj)), clueflow);
 #endif
 		}
 
-		/* Merge the first element with the previous one if possible.  */
-		if (p->prev == NULL)
-			merge_possibly (engine, obj_copy->prev, obj_copy);
-		/* Merge the last element with the following one if possible. */
-		if (p->next == NULL)
-			merge_possibly (engine, obj_copy, obj_copy->next);
+		if (p->prev == NULL) {
+			if (html_object_is_text (obj_copy))
+				merge_possibly (engine, obj_copy->prev, obj_copy);
+			else
+				remove_element_if_empty_text
+					(engine, html_object_prev_not_slave (obj_copy));
+		}
+
+		if (p->next == NULL) {
+			if (html_object_is_text (obj_copy))
+				merge_possibly (engine, obj_copy, obj_copy->next);
+			else
+				remove_element_if_empty_text
+					(engine, html_object_next_not_slave (obj_copy));
+		}
 	}
 
 #ifdef PARANOID_DEBUG
@@ -481,6 +532,8 @@ do_paste (HTMLEngine *engine,
 	/* 9. Normalize the cursor pointer.  */
 	html_cursor_normalize (engine->cursor);
 
+	g_print ("%s -- %d element(s).\n", __FUNCTION__, engine->cursor->position - oldpos);
+
 	/* Return the number of character elements pasted, to make undo/redo possible.  */
 	return engine->cursor->position - oldpos;
 }
@@ -498,6 +551,15 @@ struct _ActionData {
 
 	/* Number of character elements in the buffer.  */
 	guint buffer_count;
+
+	/* The original style of the clueflow in which we are pasting.  */
+	HTMLClueFlowStyle orig_clueflow_style;
+
+	/* The original alignment of the clueflow in which we are pasting.  */
+	HTMLHAlignType orig_halignment;
+
+	/* The original indentation level of the clueflow in which we are pasting.  */
+	gint orig_indentation;
 };
 typedef struct _ActionData ActionData;
 
@@ -561,10 +623,26 @@ static void
 do_undo (HTMLEngine *engine,
 	 gpointer closure)
 {
+	HTMLClueFlow *current_clueflow;
 	ActionData *data;
+	gint delta_indentation;
 
 	data = (ActionData *) closure;
+
 	html_engine_delete (engine, data->buffer_count, FALSE, TRUE);
+
+	current_clueflow = HTML_CLUEFLOW (engine->cursor->object->parent);
+	delta_indentation = data->orig_indentation - html_clueflow_get_indentation (current_clueflow);
+
+	html_engine_set_clueflow_style (engine,
+					data->orig_clueflow_style,
+					data->orig_halignment,
+					delta_indentation,
+					(HTML_ENGINE_SET_CLUEFLOW_STYLE
+					 | HTML_ENGINE_SET_CLUEFLOW_ALIGNMENT
+					 | HTML_ENGINE_SET_CLUEFLOW_INDENTATION),
+					FALSE);
+
 	setup_redo (engine, data);
 }
 
@@ -590,14 +668,20 @@ setup_undo (HTMLEngine *engine,
 static ActionData *
 action_data_from_cut_buffer (HTMLEngine *engine,
 			     GList *buffer,
-			     guint count)
+			     guint count,
+			     HTMLClueFlowStyle orig_clueflow_style,
+			     HTMLHAlignType orig_halignment,
+			     gint orig_indentation)
 {
 	ActionData *data;
 
 	data = g_new (ActionData, 1);
-	data->ref_count = 0;
-	data->buffer = html_engine_cut_buffer_dup (buffer);
-	data->buffer_count = count;
+	data->ref_count           = 0;
+	data->buffer              = html_engine_cut_buffer_dup (buffer);
+	data->buffer_count        = count;
+	data->orig_clueflow_style = orig_clueflow_style;
+	data->orig_halignment     = orig_halignment;
+	data->orig_indentation    = orig_indentation;
 
 	return data;
 }
@@ -635,6 +719,10 @@ void
 html_engine_paste (HTMLEngine *engine,
 		   gboolean do_undo)
 {
+	HTMLClueFlow *current_clueflow;
+	HTMLClueFlowStyle orig_clueflow_style;
+	HTMLHAlignType orig_halignment;
+	guint orig_indentation;
 	guint count;
 
 	g_return_if_fail (engine != NULL);
@@ -649,6 +737,13 @@ html_engine_paste (HTMLEngine *engine,
 		html_undo_level_begin (engine->undo, "paste");
 	}
 
+	current_clueflow = HTML_CLUEFLOW (engine->cursor->object->parent);
+	g_assert (current_clueflow != NULL);
+
+	orig_clueflow_style = html_clueflow_get_style (current_clueflow);
+	orig_halignment = html_clueflow_get_halignment (current_clueflow);
+	orig_indentation = html_clueflow_get_indentation (current_clueflow);
+
 	/* Cut current selection.  */
 	if (engine->active_selection) {
 		GList *cut_buffer;
@@ -669,7 +764,13 @@ html_engine_paste (HTMLEngine *engine,
 	count = do_paste (engine, engine->cut_buffer);
 
 	if (do_undo) {
-		setup_undo (engine, action_data_from_cut_buffer (engine, engine->cut_buffer, count));
+		ActionData *action_data;
+
+		action_data = action_data_from_cut_buffer (engine, engine->cut_buffer, count,
+							   orig_clueflow_style,
+							   orig_halignment,
+							   orig_indentation);
+		setup_undo (engine, action_data);
 		html_undo_level_end (engine->undo);
 	}
 }
@@ -693,13 +794,14 @@ html_engine_paste_object (HTMLEngine *engine,
 	g_return_if_fail (HTML_IS_ENGINE (engine));
 	g_return_if_fail (object != NULL);
 
+	/* FIXME: Nasty, nasty, nasty hack.  */
+
 	tmp_buffer = engine->cut_buffer;
 	engine->cut_buffer = g_list_alloc ();
 	engine->cut_buffer->data = object;
 
 	html_engine_paste (engine, do_undo);
 
+	g_list_free (engine->cut_buffer);
 	engine->cut_buffer = tmp_buffer;
-
-	g_list_free (tmp_buffer);
 }
