@@ -427,6 +427,13 @@ save (HTMLObject *self,
 	return TRUE;
 }
 
+static gboolean
+save_plain (HTMLObject *self,
+      HTMLEngineSaveState *state)
+{
+	return TRUE;
+}
+
 static gint
 check_page_split (HTMLObject *self,
 		  gint y)
@@ -493,6 +500,7 @@ html_object_class_init (HTMLObjectClass *klass,
 	klass->forall = forall;
 	klass->is_container = is_container;
 	klass->save = save;
+	klass->save_plain = save_plain;
 	klass->check_page_split = check_page_split;
 	klass->search = search;
 	klass->search_next = search;
@@ -756,6 +764,7 @@ html_object_relayout (HTMLObject *self,
 		      HTMLEngine *engine,
 		      HTMLObject *child)
 {
+	g_return_val_if_fail (self != NULL, TRUE);
 	return (* HO_CLASS (self)->relayout) (self, engine, child);
 }
 
@@ -894,6 +903,12 @@ html_object_save (HTMLObject *self,
 	return (* HO_CLASS (self)->save) (self, state);
 }
 
+gboolean
+html_object_save_plain (HTMLObject *self,
+			HTMLEngineSaveState *state)
+{
+	return (* HO_CLASS (self)->save_plain) (self, state);
+}
 
 gint
 html_object_check_page_split  (HTMLObject *self,
