@@ -37,6 +37,13 @@
 #define HTML_IS_PAINTER(obj)              (GTK_CHECK_TYPE ((obj), HTML_TYPE_PAINTER))
 #define HTML_IS_PAINTER_CLASS(klass)      (GTK_CHECK_CLASS_TYPE ((klass), HTML_TYPE_PAINTER))
 
+
+typedef enum {
+	GTK_HTML_ETCH_IN,
+	GTK_HTML_ETCH_OUT,
+	GTK_HTML_ETCH_NONE
+} GtkHTMLEtchStyle; 
+
 
 struct _HTMLPainter {
 	GtkObject base;
@@ -74,12 +81,12 @@ struct _HTMLPainterClass {
 	void (* set_background_color) (HTMLPainter *painter, const GdkColor *color);
 	void (* draw_shade_line) (HTMLPainter *p, gint x, gint y, gint width);
 	void (* draw_panel) (HTMLPainter *painter, gint x, gint y, gint width, gint height,
-			     gboolean inset, gint bordersize);
+			     GtkHTMLEtchStyle inset, gint bordersize);
 
 	void (* set_clip_rectangle) (HTMLPainter *painter, gint x, gint y, gint width, gint height);
-	void (* draw_background_pixmap) (HTMLPainter *painter, gint x, gint y, GdkPixbuf *pixbuf,
-					 gint pix_width, gint pix_height);
-
+	void (* draw_background) (HTMLPainter *painter, const GdkColor *color, GdkPixbuf *pixbuf,
+				  gint x, gint y, gint width, gint height, gint tile_x, gint tile_y);
+	
 	void (* set_color_set) (HTMLPainter *painter, HTMLColorSet *color_set);
 
 	guint (* get_pixel_size) (HTMLPainter *painter);
@@ -175,7 +182,7 @@ void  html_painter_draw_panel            (HTMLPainter    *painter,
 					  gint            y,
 					  gint            width,
 					  gint            height,
-					  gboolean        inset,
+					  GtkHTMLEtchStyle inset,
 					  gint            bordersize);
 
 /* Passing 0 for width/height means remove clip rectangle */
@@ -186,13 +193,18 @@ void  html_painter_set_clip_rectangle  (HTMLPainter *painter,
 					gint         height);
 
 /* Passing 0 for pix_width / pix_height makes it use the image width */
-void  html_painter_draw_background_pixmap  (HTMLPainter *painter,
-					    gint         x,
-					    gint         y,
-					    GdkPixbuf   *pixbuf,
-					    gint         pix_width,
-					    gint         pix_height);
+void  html_painter_draw_background  (HTMLPainter *painter,
+				     const GdkColor *color,
+				     GdkPixbuf   *pixbuf,
+				     gint         x,
+				     gint         y,
+				     gint         width,
+				     gint         height,
+				     gint         tile_x,
+				     gint         tile_y);
 
 guint  html_painter_get_pixel_size  (HTMLPainter *painter);
 
 #endif /* _HTMLPAINTER_H_ */
+
+
