@@ -264,6 +264,24 @@ get_font_style (HTMLPainter *painter)
 	return printer->font_style;
 }
 
+static void
+do_rectangle (GnomePrintContext *context,
+	      gdouble x, gdouble y,
+	      gdouble width, gdouble height,
+	      gdouble line_width)
+{
+	gnome_print_setlinewidth (context, line_width);
+
+	gnome_print_newpath (context);
+	gnome_print_moveto (context, x, y);
+	gnome_print_lineto (context, x + width, y);
+	gnome_print_lineto (context, x + width, y - height);
+	gnome_print_lineto (context, x, y - height);
+	gnome_print_lineto (context, x, y);
+
+	gnome_print_stroke (context);
+}
+
 
 /* HTMLPainter drawing functions.  */
 
@@ -282,6 +300,8 @@ draw_line (HTMLPainter *painter,
 	engine_coordinates_to_gnome_print (printer, x1, y1, &printer_x1, &printer_y1);
 	engine_coordinates_to_gnome_print (printer, x2, y2, &printer_x2, &printer_y2);
 
+	gnome_print_setlinewidth (printer->print_context, PIXEL_SIZE);
+
 	gnome_print_newpath (printer->print_context);
 	gnome_print_moveto (printer->print_context, printer_x1, printer_y1);
 	gnome_print_lineto (printer->print_context, printer_x2, printer_y2);
@@ -306,21 +326,16 @@ draw_rect (HTMLPainter *painter,
 
 	engine_coordinates_to_gnome_print (printer, x, y, &printer_x, &printer_y);
 
-	gnome_print_newpath (printer->print_context);
-	gnome_print_moveto (printer->print_context, printer_x, printer_y);
-	gnome_print_lineto (printer->print_context, printer_x + printer_width, printer_y);
-	gnome_print_lineto (printer->print_context, printer_x + printer_width, printer_y - printer_height);
-	gnome_print_lineto (printer->print_context, printer_x, printer_y - printer_height);
-	gnome_print_lineto (printer->print_context, printer_x, printer_y);
-
-	gnome_print_stroke (printer->print_context);
+	do_rectangle (printer->print_context,
+		      printer_x, printer_y, printer_width, printer_height,
+		      PIXEL_SIZE);
 }
 
 static void
 draw_panel (HTMLPainter *painter,
 	    gint x, gint y,
 	    gint width, gint height,
-	    gboolean inset,
+	    GtkHTMLEtchStyle inset,
 	    gint bordersize)
 {
 	HTMLPrinter *printer;
@@ -335,21 +350,18 @@ draw_panel (HTMLPainter *painter,
 
 	engine_coordinates_to_gnome_print (printer, x, y, &printer_x, &printer_y);
 
-	gnome_print_newpath (printer->print_context);
-	gnome_print_moveto (printer->print_context, printer_x, printer_y);
-	gnome_print_lineto (printer->print_context, printer_x + printer_width, printer_y);
-	gnome_print_lineto (printer->print_context, printer_x + printer_width, printer_y - printer_height);
-	gnome_print_lineto (printer->print_context, printer_x, printer_y - printer_height);
-	gnome_print_lineto (printer->print_context, printer_x, printer_y);
-
-	gnome_print_stroke (printer->print_context);
+	do_rectangle (printer->print_context,
+		      printer_x, printer_y, printer_width, printer_height,
+		      bordersize * PIXEL_SIZE);
 }
 
 static void
-draw_background_pixmap (HTMLPainter *painter,
-			gint x, gint y, 
-			GdkPixbuf *pixbuf,
-			gint pix_width, gint pix_height)
+draw_background (HTMLPainter *painter,
+		 const GdkColor *color,
+		 GdkPixbuf *pixbuf,
+		 gint x, gint y, 
+		 gint pix_width, gint pix_height,
+		 gint tile_x, gint tile_y)
 {
 	HTMLPrinter *printer;
 
@@ -644,7 +656,7 @@ class_init (GtkObjectClass *object_class)
 	painter_class->draw_pixmap = draw_pixmap;
 	painter_class->clear = clear;
 	painter_class->draw_shade_line = draw_shade_line;
-	painter_class->draw_background_pixmap = draw_background_pixmap;
+	painter_class->draw_background = draw_background;
 	painter_class->get_pixel_size = get_pixel_size;
 
 	parent_class = gtk_type_class (html_painter_get_type ());
