@@ -27,6 +27,8 @@ HTMLTextInputClass html_text_input_class;
 static HTMLEmbeddedClass *parent_class = NULL;
 
 
+/* HTMLObject methods.  */
+
 static void
 destroy (HTMLObject *o)
 {
@@ -37,7 +39,7 @@ destroy (HTMLObject *o)
 	if (ti->default_text)
 		g_free (ti->default_text);
 
-	HTML_OBJECT_CLASS (&html_embedded_class)->destroy (o);
+	HTML_OBJECT_CLASS (parent_class)->destroy (o);
 }
 
 static void
@@ -60,6 +62,9 @@ reset (HTMLEmbedded *e)
 	gtk_entry_set_text (GTK_ENTRY(e->widget), HTML_TEXTINPUT(e)->default_text);
 }
 
+
+/* HTMLEmbedded methods.  */
+
 static gchar *
 encode (HTMLEmbedded *e)
 {
@@ -84,10 +89,13 @@ encode (HTMLEmbedded *e)
 	return ptr;
 }
 
+
 void
 html_text_input_type_init (void)
 {
-	html_text_input_class_init (&html_text_input_class, HTML_TYPE_TEXTINPUT, sizeof (HTMLTextInput));
+	html_text_input_class_init (&html_text_input_class,
+				    HTML_TYPE_TEXTINPUT,
+				    sizeof (HTMLTextInput));
 }
 
 void
@@ -103,14 +111,14 @@ html_text_input_class_init (HTMLTextInputClass *klass,
 
 	html_embedded_class_init (element_class, type, object_size);
 
-	/* HTMLEmbedded methods.   */
-	element_class->reset = reset;
-	element_class->encode = encode;
-
 	/* HTMLObject methods.   */
 	object_class->destroy = destroy;
 	object_class->copy = copy;
 
+	/* HTMLEmbedded methods.   */
+	element_class->reset = reset;
+	element_class->encode = encode;
+
 	parent_class = &html_embedded_class;
 }
 
@@ -127,7 +135,7 @@ html_text_input_init (HTMLTextInput *ti,
 
 	HTMLEmbedded *element;
 	HTMLObject *object;
-	GtkRequisition req;
+	gint min_width;
 
 	element = HTML_EMBEDDED (ti);
 	object = HTML_OBJECT (ti);
@@ -135,29 +143,19 @@ html_text_input_init (HTMLTextInput *ti,
 	html_embedded_init (element, HTML_EMBEDDED_CLASS (klass),
 			   parent, name, value);
 
-	element->widget = gtk_entry_new();
-	gtk_widget_size_request(element->widget, &req);
-
-	if(strlen (element->value))	
-		gtk_entry_set_text(GTK_ENTRY(element->widget), element->value);
+	html_embedded_set_widget (element, gtk_entry_new());
+	if (strlen (element->value))	
+		gtk_entry_set_text (GTK_ENTRY(element->widget), element->value);
 
 	ti->default_text = g_strdup (element->value);
 
-	if(maxlen != -1)
+	if (maxlen != -1)
 		gtk_entry_set_max_length(GTK_ENTRY(element->widget), maxlen);
 
 	gtk_entry_set_visibility (GTK_ENTRY(element->widget), !password);
 	
-	req.width = gdk_char_width(element->widget->style->font, '0') * size + 8;
-
-	gtk_widget_set_usize(element->widget, req.width, req.height);
-
-	object->descent = 0;
-	object->width = req.width;
-	object->ascent = req.height;
-
-	/*	gtk_widget_show(element->widget);
-		gtk_layout_put(GTK_LAYOUT(parent), element->widget, 0, 0);*/
+	min_width = gdk_char_width (element->widget->style->font, '0') * size + 8;
+	gtk_widget_set_usize (element->widget, min_width, -1);
 
 	ti->size = size;
 	ti->maxlen = maxlen;
