@@ -413,6 +413,9 @@ get_cursor_base (HTMLObject *self,
 			return;
 		}
 	}
+
+	g_warning ("Getting cursor base for an HTMLTextMaster with no slaves -- %p\n",
+		   self);
 }
 
 
@@ -543,6 +546,12 @@ merge (HTMLText *self,
 	if (other->text_len == 0)
 		return;
 
+	if (self->text_len == 0) {
+		self->font_style = other->font_style;
+		self->color = other->color;
+		self->color_allocated = other->color_allocated;
+	}
+
 	if (prepend)
 		calculate_new_selection (other, self, &select_start, &select_length);
 	else
@@ -574,6 +583,8 @@ merge (HTMLText *self,
 		HTML_OBJECT (self)->selected = TRUE;
 	else
 		HTML_OBJECT (self)->selected = FALSE;
+
+	html_object_change_set (HTML_OBJECT (self), HTML_CHANGE_ALL);
 }
 
 static guint
@@ -742,3 +753,21 @@ html_text_master_trail_space_width (HTMLTextMaster *master, HTMLPainter *painter
 		return 0;
 	}
 }
+
+
+void
+html_text_master_destroy_slaves (HTMLTextMaster *master)
+{
+	HTMLObject *p;
+	HTMLObject *pnext;
+
+	g_return_if_fail (master != NULL);
+
+	for (p = HTML_OBJECT (master)->next;
+	     p != NULL && HTML_OBJECT_TYPE (p) == HTML_TYPE_TEXTSLAVE;
+	     p = pnext) {
+		pnext = p->next;
+		html_clue_remove (HTML_CLUE (p->parent), p);
+		html_object_destroy (p);
+	}
+}
